@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -39,12 +39,11 @@ from hypothesis._settings import (
     Verbosity,
     note_deprecation,
     settings as Settings,
-    unlimited,
 )
 from hypothesis.control import current_build_context
 from hypothesis.core import given
-from hypothesis.errors import InvalidArgument, InvalidDefinition
-from hypothesis.internal.compat import int_to_bytes, string_types
+from hypothesis.errors import HypothesisException, InvalidArgument, InvalidDefinition
+from hypothesis.internal.compat import quiet_raise, string_types
 from hypothesis.internal.reflection import function_digest, nicerepr, proxies
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
@@ -83,11 +82,7 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
             settings = state_machine_factory.TestCase.settings
             check_type(Settings, settings, "state_machine_factory.TestCase.settings")
         except AttributeError:
-            settings = Settings(
-                timeout=unlimited,
-                deadline=None,
-                suppress_health_check=HealthCheck.all(),
-            )
+            settings = Settings(deadline=None, suppress_health_check=HealthCheck.all())
     check_type(Settings, settings, "settings")
 
     @settings
@@ -211,11 +206,9 @@ class GenericStateMachine(
 
         Does nothing by default.
         """
-        pass
 
     def check_invariants(self):
         """Called after initializing and after executing each step."""
-        pass
 
     _test_case_cache = {}  # type: dict
 
@@ -229,11 +222,7 @@ class GenericStateMachine(
             pass
 
         class StateMachineTestCase(TestCase):
-            settings = Settings(
-                timeout=unlimited,
-                deadline=None,
-                suppress_health_check=HealthCheck.all(),
-            )
+            settings = Settings(deadline=None, suppress_health_check=HealthCheck.all())
 
         # We define this outside of the class and assign it because you can't
         # assign attributes to instance method values in Python 2
@@ -331,6 +320,21 @@ def consumes(bundle):
     return BundleConsumer(bundle)
 
 
+@attr.s()
+class MultipleResults(object):
+    values = attr.ib()
+
+
+def multiple(*args):
+    """This function can be used to pass multiple results to the target(s) of
+    a rule. Just use ``return multiple(result1, result2, ...)`` in your rule.
+
+    It is also possible to use ``return multiple()`` with no arguments in
+    order to end a rule without passing any result.
+    """
+    return MultipleResults(args)
+
+
 def _convert_targets(targets, target):
     """Single validator and convertor for target arguments."""
     if target is not None:
@@ -589,42 +593,12 @@ class RuleStrategy(SearchStrategy):
         )
 
     def do_draw(self, data):
-        # This strategy is slightly strange in its implementation.
-        # We don't want the interpretation of the rule we draw to change based
-        # on whether other rules satisfy their preconditions or have data in
-        # their bundles. Therefore the index into the rule list needs to stay
-        # stable. BUT we don't want to draw invalid rules. So what we do is we
-        # draw an index. We *could* just loop until it's valid, but if most
-        # rules are invalid then that could result in a very long loop.
-        # So what we do is the following:
-        #
-        #   1. We first draw a rule unconditionally, and check if it's valid.
-        #      If it is, great. Nothing more to do, that's our rule.
-        #   2. If it is invalid, we now calculate the list of valid rules and
-        #      draw from that list (if there are none, that's an error in the
-        #      definition of the machine and we complain to the user about it).
-        #   3. Once we've drawn a valid rule, we write that back to the byte
-        #      stream. As a result, when shrinking runs the shrinker can delete
-        #      the initial failed draw + the draw that lead to us finding an
-        #      index into valid_rules, leaving just the written value of i.
-        #      When this is run, it will look as we got lucky and just happened
-        #      to pick a valid rule.
-        #
-        # Easy, right?
-        n = len(self.rules)
-        i = cu.integer_range(data, 0, n - 1)
-        u, v = data.blocks[-1].bounds
-        block_length = v - u
-        rule = self.rules[i]
-        if not self.is_valid(rule):
-            valid_rules = [j for j, r in enumerate(self.rules) if self.is_valid(r)]
-            if not valid_rules:
-                raise InvalidDefinition(
-                    u"No progress can be made from state %r" % (self.machine,)
-                )
-            i = valid_rules[cu.integer_range(data, 0, len(valid_rules) - 1)]
-            data.write(int_to_bytes(i, block_length))
-            rule = self.rules[i]
+        try:
+            rule = data.draw(st.sampled_from(self.rules).filter(self.is_valid))
+        except HypothesisException:
+            # FailedHealthCheck or UnsatisfiedAssumption depending on user settings.
+            msg = u"No progress can be made from state %r" % (self.machine,)
+            quiet_raise(InvalidDefinition(msg))
         return (rule, data.draw(rule.arguments_strategy))
 
     def is_valid(self, rule):
@@ -795,6 +769,15 @@ class RuleBasedStateMachine(GenericStateMachine):
             )
         )
 
+    def _add_result_to_targets(self, targets, result):
+        name = self.new_name()
+        self.__printer.singleton_pprinters.setdefault(
+            id(result), lambda obj, p, cycle: p.text(name)
+        )
+        self.names_to_values[name] = result
+        for target in targets:
+            self.bundle(target).append(VarReference(name))
+
     def execute_step(self, step):
         rule, data = step
         data = dict(data)
@@ -803,13 +786,11 @@ class RuleBasedStateMachine(GenericStateMachine):
                 data[k] = self.names_to_values[v.name]
         result = rule.function(self, **data)
         if rule.targets:
-            name = self.new_name()
-            self.names_to_values[name] = result
-            self.__printer.singleton_pprinters.setdefault(
-                id(result), lambda obj, p, cycle: p.text(name)
-            )
-            for target in rule.targets:
-                self.bundle(target).append(VarReference(name))
+            if isinstance(result, MultipleResults):
+                for single_result in result.values:
+                    self._add_result_to_targets(rule.targets, single_result)
+            else:
+                self._add_result_to_targets(rule.targets, result)
         if self._initialize_rules_to_run:
             self._initialize_rules_to_run.remove(rule)
 
